/*
    Form.java by Frank B. Brokken (f.b.brokken@rc.rug.nl), June 2001
    Distribution: GPL

    Since all local  developments are stored in the package local,
    there is no need for further Dutch names. 

    To avoid ugly language combinations, I now switch to English as the
    primary language.

    ---------------------------
    
    Class Form defines a basic class for form based input/output. It uses
    the GridBagLayout, so all layout rows and cols are LOGICAL, rather than
        GRIDWISE. I.e., an all-empty column or all-empty row does not exist.


    See the file doc/form.{html,tex,txt,yo} for details about this class.

*/

package local;

import local.form.*;

import java.lang.*;
import java.awt.*;
import java.awt.event.*;

public class Form extends Frame implements Runnable
{
    public static final int PACKED = 1;

    public static final int CENTER = GridBagConstraints.CENTER;
    public static final int NORTH = GridBagConstraints.NORTH;
    public static final int NORTHEAST = GridBagConstraints.NORTHEAST;
    public static final int EAST = GridBagConstraints.EAST;
    public static final int SOUTHEAST = GridBagConstraints.SOUTHEAST;
    public static final int SOUTH = GridBagConstraints.SOUTH;
    public static final int SOUTHWEST = GridBagConstraints.SOUTHWEST;
    public static final int WEST = GridBagConstraints.WEST;
    public static final int NORTHWEST = GridBagConstraints.NORTHWEST;

    public static final int NONE = GridBagConstraints.NONE;
    public static final int BOTH = GridBagConstraints.BOTH;
    public static final int HORIZONTAL = GridBagConstraints.HORIZONTAL;
    public static final int VERTICAL = GridBagConstraints.VERTICAL;

    public Form(int xsize, int ysize, String title, Color fg, Color bg)
    {
        super(title);
        bgcolor = bg;
        fgcolor = fg;

        panel = new Panel(gridbag);
        panel.setForeground(fgcolor);
        panel.setBackground(bgcolor);

        this.xsize = xsize;
        this.ysize = ysize;
    }

    public Form(int xsize, int ysize, String title)
    {
        this(xsize, ysize, title, Color.black, Color.gray);
    }

    public void nextRow()
    {
        ++row;
        col = 0;
    }

    public void nextCol()
    {
        ++col;
    }

    public void nextWidth(int width)
    {
        constraints.gridwidth = width;
    }

    public void nextHeight(int height)
    {
        constraints.gridheight = height;
    }

    public void nextAnchor(int anchor)
    {
        constraints.anchor = anchor;
    }

    public void setAnchor(int anchor)
    {
        nextAnchor(anchor);
        this.anchor = anchor;
    }

    public void nextFill(int fill)
    {
        constraints.fill = fill;
    }

    public void setFill(int fill)
    {
        nextFill(fill);
        this.fill = fill;
    }

    public void setBackground(Color bgcolor)
    {
        this.bgcolor = bgcolor;
    }

    public void setForeground(Color fgcolor)
    {
        this.fgcolor = fgcolor;
    }

    public void display()
    {
        display(0);
    }

    public void display(int packMode)
    {
        if (thread == null)
        {
            thread = new PackedThread(packMode, this);
            thread.start();
        }
    }

    public Button addButton(String label, 
                    ButtonHandler handler, Color fg, Color bg)
    {
        Button
            b = new Button(label);

        b.setForeground(fg);
        b.setBackground(bg);
        b.addActionListener(new ButtonListener(b, handler));
        setConstraints(b);
        panel.add(b);
        return b;
    }

    public Button addButton(String label, ButtonHandler handler)
    {
        return addButton(label, handler, fgcolor, bgcolor);
    }

    public Label addLabel(String text, Color fg, Color bg)
    {
        Label
            l = new Label(text);
        l.setForeground(fg);
        l.setBackground(bg);
        setConstraints(l);
        panel.add(l);
        return l;
    }

    public Label addLabel(String text)
    {
        return addLabel(text, fgcolor, bgcolor);
    }

    public TextField addInputLine(String accept,
                int width, int max, TextHandler handler,
                Color fg, Color bg)
    {
        TextField
            t = new TextField(handler.getText(), width);
        t.setForeground(fg);
        t.setBackground(bg);

        LocalTextListener
            tl = new LocalTextListener(t, handler);

        t.addActionListener(tl);
        t.addFocusListener(tl);

        if (accept.length() > 0)
            t.addKeyListener(new KeyFilter(accept));

        t.addKeyListener(new TextLengthLimiter(max));

        setConstraints(t);
        panel.add(t);
        return t;
    }

    public TextField addInputLine(int width, int max, TextHandler buffer,
                            Color fg, Color bg)
    {
        return addInputLine("", width, max, buffer, fg, bg);
    }

    public TextField addInputLine(int width, int max, TextHandler buffer)
    {
        return addInputLine("", width, max, buffer, fgcolor, bgcolor);
    }

    public TextField addInputLine(String accept, 
                            int width, int max, TextHandler buffer)
    {
        return addInputLine(accept, width, max, buffer, fgcolor, bgcolor);
    }


    public TextArea  addInputArea(int cols, int rows, TextHandler handler,
                Color fg, Color bg)
    {
        TextArea
            t = new TextArea(handler.getText(), rows, cols);
        t.setForeground(fg);
        t.setBackground(bg);

        LocalTextListener
            tl = new LocalTextListener(t, handler);

        t.addFocusListener(tl);

        setConstraints(t);
        panel.add(t);
        return t;
    }


    public TextArea addInputArea(int cols, int rows, TextHandler handler)
    {
        return addInputArea(cols, rows, handler, fgcolor, bgcolor);
    }

    public Canvas addCanvas(CanvasHandler handler, 
                            Color fgcolor, Color bgcolor)
    {
        Panel
            p = new Panel();

        p.setForeground(fgcolor);
        p.setBackground(bgcolor);

        p.add(handler);

        setConstraints(p);
        panel.add(p);
        return handler;
    }

    public Canvas addCanvas(CanvasHandler handler)
    {
        return addCanvas(handler, bgcolor, fgcolor);
    }

    public Panel addPanel(Panel p, Color fgcolor, Color bgcolor)
    {
        p.setForeground(fgcolor);
        p.setBackground(bgcolor);

        setConstraints(p);
        panel.add(p);
        return p;
    }

    public Component addComponent(Component component)
    {
        return addComponent(component, fgcolor, bgcolor);
    }

    public Component addComponent(Component component, 
                                    Color fgcolor, Color bgcolor)
    {
        component.setForeground(fgcolor);
        component.setBackground(bgcolor);

        setConstraints(component);
        panel.add(component);
        return component;
    }

    public void run()
    {
        add(panel);
        setSize(xsize, ysize);
        super.setVisible(true);

        if (thread.getPackMode() == PACKED)
            pack();
    }

    private void setConstraints(Component component)
    {
        constraints.gridx = col++;
        constraints.gridy = row;

        gridbag.setConstraints(component, constraints);

        constraints.fill = fill;
        constraints.anchor = anchor;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
    }

    private Color
        bgcolor,
        fgcolor;
    private PackedThread
        thread;
    private Panel
        panel;
    private int
        fill = NONE,
        anchor = EAST, //CENTER,
        row = 0,
        col = 0,
        xsize,
        ysize;
    private GridBagLayout
        gridbag = new GridBagLayout();
    private GridBagConstraints
        constraints = new GridBagConstraints();
}





