/*
    ExtractorBuffer.java by Frank B. Brokken (f.b.brokken@rc.rug.nl), June 2001
    Distribution: GPL

    Since all local  developments are stored in the package local,
    there is no need for further Dutch names. 

    To avoid unly language combinations, I now switch to English as the
    primary language.

    ---------------------------

    This class is an auxiliary class for Extractor. It defines a buffer for
all input operations, allowing a reset to the state corresponding to the last
successful read.

    It defines the following members:

    ExtractorBuffer(InputStreamReader reader):    basic constructor

    String substring(int begin, int end)        return substring 
                                                [begin, end) from the buffer

    void flush()                                remove processed chars from
                                                the buffer

    void reset()                                allow non-processed chars to
                                                be reread. State to OK

    void pushback(String s)                     insert a string so that
                                                it will be read next

    int state()                                 return the ExtractorBuffer 
                                                state

    int end()                                   returns the first free index
                                                in the buffer    

    String message()                            returns last message

    char readChar()                             returns the next available
                                                character or 0

    int fillLine()                              returns the index beyond
                                                the next available line or -1

    int fillChar(char c)                        returns idx beyond `c' or -1

    int fillRange(char first, char last)        returns idx beyond inclusive 
                                                range or -1

    int fillWhitespace()                        returns idx beyond whitespace 
                                                or -1

    int fillNonWhitespace()                     returns idx at ws or -1

    void close()                                close the reader

    static final int EXCEPTION = -2;            state value for an Exception
    static final int EOF = -1;                  state value for EOF
    static final int OK = 0;                    state value for all OK

    // ******** Private members

    private void readIfEmpty()                  reads 1 char into the buffer
                                                unless characters wait for 
                                                processing

    private boolean read()                      read the next character in the 
                                                buffer, return true or return
                                                false. 

    private boolean eoln()                      check EOF condition: \r or \n 
                                                in the buffer. 

    private boolean within(char first, char last)   true if chars within range

*/

package local.ios;

import java.io.*;

public class ExtractorBuffer
{
    public ExtractorBuffer(InputStreamReader reader)
    {
        this.reader = reader;
        buffer = new String();
        beyond = 0;
        theState = OK;
    }

    public String substring(int begin, int end) // return substring from the
    {                                           // buffer.
        return buffer.substring(begin, end);
    }

    public void flush()                         // remove [0, beyond) from the
    {                                           // buffer, reset beyond to 0
        buffer = buffer.substring(beyond);
        beyond = 0;
    }

    public void reset()                         // set beyond to 0, keep the
    {                                           // buffer, set state() to OK
        beyond = 0;
        theState = OK;
    }

    public int state()                          // return the ExtractorBuffer
    {                                           // state
        return theState;
    }

    public int end()                            // returns the first free
    {                                           // index in the buffer    
        return beyond;
    }

    public String message()                     // return last set message
    {
        return msg;
    }

    public void pushback(String string)
    {
        buffer = 
            beyond == 0 ? 
                string + buffer
            :
                buffer.substring(0, beyond) + 
                string + buffer.substring(beyond);
    }

    public char readChar()                      // returns the next available
    {                                           // character or 0
        readIfEmpty();
        return (beyond < buffer.length()) ? buffer.charAt(beyond++) : 0;
    }

    public int fillLine()                       // returns the index beyond
    {                                           // the next available line
        while (beyond < buffer.length())        // first inspect available
        {                                       // chars
            if (eoln())                         // done at eoln() indicator
                return beyond;
            beyond++;
        }
        while (read())                          // otherwise read chars
        {
            if (eoln())                         // done at eoln
                return beyond;
            beyond++;
        }

        if (beyond > 0)                         // anything read ?
            return beyond;                      // tell the caller    

        return -1;                              // no line was read.
    }

    public int fillChar(char c)                 // return -1 or idx beyond `c'
    {
        readIfEmpty();
        if (beyond < buffer.length())
        {
            if (buffer.charAt(beyond) == c) 
                ++beyond;
            return beyond;
        }
        return -1;            
    }

    public int fillRange(char first, char last) // idx beyond inclusive range
    {                                           // or -1
        while (beyond < buffer.length())
        {
            if (!within(first, last))
                return beyond;
            beyond++;
        }
        while (read())
        {
            if (!within(first, last))
                return beyond;
            beyond++;
        }
        return -1;  
    }
            
    public int fillWhitespace()                  // idx beyond whitespace or -1
    {
        while (beyond < buffer.length())
        {
            if (!within(asciiZ, ' '))
                return beyond;
            beyond++;
        }
        while (read())
        {
            if (!within(asciiZ, ' '))
                return beyond;
            beyond++;
        }
        return -1;  
    }          

    public int fillNonWhitespace()              // idx at ws or -1
    {
        while (beyond < buffer.length())
        {
            if (within(asciiZ, ' '))
                return beyond;
            beyond++;
        }
        while (read())
        {
            if (within(asciiZ, ' '))
                return beyond;
            beyond++;
        }
        return -1;  
    }          

    public static final int EXCEPTION = -2;           
    public static final int EOF = -1;           
    public static final int OK = 0;

    public void close()                        // close the reader
    {
        try
        {
            reader.close();
        }
        catch (Exception e)
        {
            msg = e.toString();
            theState = EXCEPTION;
        }
    }

    // ******** Private

    private void readIfEmpty()
    {   
        if (beyond == buffer.length())
            read();
    }

    private boolean read()              // read the next character in the 
    {                                   // buffer, return true or return
                                        // false. 
        try
        {
            int
                ch = reader.read();

            if (ch != -1)
            {
                buffer = buffer + String.valueOf((char)ch);
                theState = OK;
                return true;
            }
            theState = EOF;             // when -1 is read, EOF is true.
        }
        catch (Exception e)
        {
            msg = e.toString();
            theState = EXCEPTION;
        }
        return false;
    }

                // the following functions are called AFTER reading a 
                // character, and BEFORE updating beyond, so beyond
                // inexes the last inspected character. Not necessarily the
                // last read character, as a reset might have set beyond to 0,
                // so we can't use a `lastRead' character or similar.

    private boolean eoln()                  // check EOF condition: \r or \n 
    {                                       // in the buffer. 
        int
            ch = buffer.charAt(beyond);

        return ch == '\n' || ch == '\r';
    }

    private boolean within(char first, char last)
    {
        char
            ch = buffer.charAt(beyond);

        return first <= ch && ch <= last;
    }

            
    private InputStreamReader 
        reader;
    private String
        buffer,
        msg;
    private int 
        theState,
        beyond;
    private final char asciiZ = (char)0;
}
