/*
    Extractor.java by Frank B. Brokken (f.b.brokken@rc.rug.nl), June 2001
    Distribution: GPL

    Since all local  developments are stored in the package local,
    there is no need for further Dutch names. 

    To avoid unly language combinations, I now switch to English as the
    primary language.

    ---------------------------
    
    Class Extractor defines the class for text-file based input using
    InputStream objects

    It defines the following members:

    Extractor()   -   Basic constructor, used for making available System.in
    
    char readChar()     - returns a char, 0 on failure.

    String readLine()   - retuns a line, until \n (not included)
                            fail() is true if no \n was found
                            (e.g., the last line of a file, not terminated by
                            \n), but the line itself is returned.
                            null on failure (e.g., eof()).

    eating initial whitespace:

    boolean readBoolean() - returns a boolean; false on failure.

    int readInt()       - returns an int; Integer.MIN_VALUE on failure
    int readLong()      - return an long; Long.MIN_VALUE on failure
    double readFloat()  - returns a float; Float.NEGATIVE_INFINITY on failure
    double readDouble() - returns a double; Double.NEGATIVE_INFINITY on failure
    String readString() - returns a String; null on failure

    clear() will reset the input buffer on read-failures.

    private members:

    open(InputStream)   - constructs a wrapper around an existing 
                            InputStream 
                               
*/
    
package local;

import java.io.*;

public class Extractor extends ios 
{
    public Extractor()            // basic constructor, opening System.in
    {          
        open(new InputStreamReader(System.in));
    }

    public char readChar()      // read a single character, 0 on failure
    {
        if (!good)
            return 0;

        int
            ch = buffer.readChar();

        switch (buffer.state())   // error states: return 0
        {
            case ExtractorBuffer.EOF:
                setEof();
                return 0;
            case ExtractorBuffer.EXCEPTION:
                setMsg(buffer.message());
                return 0;
        }

        buffer.flush();           // flush the input, and return the char
        return (char)ch;
    }

    public String readLine()        
    {
        if (!good)
            return null;

        int                             // get the next line idx
            idx = buffer.fillLine();

        if (idx != -1)                  // got it
        {
            String                      // fetch the string
                s = buffer.substring(0, idx);

            readChar();                 // and skip the \n. If this results
                                        // in EOF, then the appropriate 
                                        // state is set by readChar

            buffer.flush();           // flush anyway, since the line
                                        // itself was read

            return s;                   // but the line is always returned    
        }
                                        // line reading failed: set the state
        if (buffer.state() == buffer.EOF)   
            setEof();                   // to EOF
        else
            setMsg(buffer.message()); // or to whatever buffer tells us

        return null;
    }

    public String readString()      
    {
        return readString(true);        // flush if string was read
    }

    public long readLong()      
    {
        if (!good)
            return Long.MIN_VALUE;

        int
            postWs = buffer.fillWhitespace();     // skip WS

        buffer.fillChar('-');              // get an optional -
        
        int                                  // get all digits
            postLong = buffer.fillRange('0', '9');

        if (postLong != -1)
        {
            String
                longString = buffer.substring(postWs, postLong);
            try
            {
                long
                    value = Long.parseLong(longString);

                buffer.flush();
                return value;
            }
            catch(Exception e)
            {}
        }
        
        setMsg("<long> expected");
        return Long.MIN_VALUE;
    }

    public int readInt()      
    {
        if (!good)
            return Integer.MIN_VALUE;

        long
            longValue = readLong();

        if (!good)
        {
            setMsg("<int> expected");
            return Integer.MIN_VALUE;
        }

        return (int)longValue;
    }

    public double readDouble()      
    {
        if (!good)
            return Double.NEGATIVE_INFINITY;

        int
            postWs = buffer.fillWhitespace();

        buffer.fillChar('-');    
        
        int    
            intpart = buffer.fillRange('0', '9'),
            point = buffer.fillChar('.'),
            fractpart = buffer.fillRange('0', '9');
        
        if (intpart  != -1 || (point != -1 && fractpart != -1))
        {
            String
                doubleString = buffer.substring(postWs, buffer.end());
            try
            {
                double
                    value = Double.valueOf(doubleString).doubleValue();

                buffer.flush();
                return value;
            }
            catch(Exception e)
            {}
        }
        
        setMsg("<double> expected");
        return  Double.NEGATIVE_INFINITY;
    }

    public float readFloat()      
    {
        if (!good)
            return Float.NEGATIVE_INFINITY;

        double
            doubleValue = readDouble();

        if (!good)
        {
            setMsg("<float> expected");
            return Float.NEGATIVE_INFINITY;
        }

        return (float)doubleValue;
    }

    public boolean readBoolean()
    {
        if (!good)
            return false;

        String
            str = readString(false);        // don't flush

        if (good)
        {
            str = str.toLowerCase();
            if (str.equals("true"))
            {
                buffer.flush();
                return true;
            }
            if (str.equals("false"))
            {
                buffer.flush();
                return false;
            }
        }

        setMsg("<boolean> expected");
        return false;
    }            

    public void clear()
    {
        super.clear();
        buffer.reset();
    }

    // ********* Protected

    protected void open(InputStreamReader inReader) 
    {
        buffer = new ExtractorBuffer(inReader);
    }

    protected ExtractorBuffer buffer;

    // ********* Private

    private String readString(boolean flush)      
    {
        if (!good)
            return null;

        int                             //  skip blanks, then the non-blanks
            postWs = buffer.fillWhitespace(),
            postString = buffer.fillNonWhitespace();

        if (postString != -1)           // got the string
        {
            String                      // retrieve the string
                s = buffer.substring(postWs, postString);

            if (flush)
                buffer.flush();         // and flush the buffer    
            return s;
        }
        
        setMsg("<String> expected");
        return null;
    }

}
   

















